function SARrot = polsar_covmat_rotate_rowcol(SAR,theta,varargin)

% Rotate full/quad polSAR data (slow for-loop implementation)
%
% Input
%   SAR   - covariance matrix representation, must be 3-D with
%             nine (9) (quad/full polarimetry) bands,
%             order is alphabetical in indexes with
%             real parts before imaginary parts (as read by
%             freadenvisar), for full polarimetry:
%               hhhh, re(hhhv), im(hhhv), re(hhvv), im(hhvv),
%               hvhv, re(hvvv), im(hvvv), vvvv
%   theta - rotation angle in degrees (theta from rho_llrr)
%   bias  - constant bias angle in degrees - optional, defaults to 0
%
%   Covariance matrix representation
%
%   ShhShh* ShhShv* ShhSvv*     k    a   rho
%   ShvShh* ShvShv* ShvSvv*  =  a*   ksi b
%   SvvShh* SvvShv* SvvSvv*     rho* b*  zeta%
%
% See also rho_llrr, rho_rrll
%
% References
%
% J.-S. Lee and D. L. Schuler and T. L. Ainsworth and E. Krogager and
% D. Kasilingam and W.-M. Boerner
% "On the estimation of radar polarization orientation shifts induced by
% terrain slopes"
% IEEE Transactions on Geoscience and Remote Sensing 40(1), 30-41, Jan. 2002
% doi:10.1109/36.981347
%
% T. L. Ainsworth, D. L. Schuler and J.-S. Lee
% "Polarimetric SAR characterization of man-made structures in urban areas
% using normalized circular-pol correlation coefficients"
% Remote Sensing of Environment, 112, 2876-2885, 2008
% doi:10.1016/j.rse.2008.02.005
%
% M. Sugimoto, K. Ouchi and Y. Nakamura
% "Four-Component Scattering Power Decomposition Algorithm with Rotation
% of Covariance Matrix Using ALOS-PALSAR Polarimetric Data",
% Remote Sensing 2012, 4, 2199-2209
% doi:10.3390/rs4082199
%
% Allan A. Nielsen
% "Rotation of Polarimetric SAR Covariance Matrices",
% IEEE International Geoscience and Remote Sensing Symposium, IGARSS 2023,
% Pasadena, California, USA, 16-21 July 2023.
% https://orbit.dtu.dk/en/publications/rotation-of-polarimetric-sar-covariance-matrices

% (c) Copyright 2022
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, people.compute.dtu.dk/alan
% 4 Mar 2022

if nargin>2
    bias = varargin{1};
    theta = theta + bias;
end

pX = size(SAR);
nrows = pX(1);
ncols = pX(2);
if  ndims(SAR)==3
    nvars = pX(3);
elseif ismatrix(SAR)
    nvars = 1;
else
    error('polsar_covmat_rotate: wrong size of input images');
end

if nvars~=9 % full/quad pol
    error('polsar_covmat_rotate: input must be full/quad pol')
end

s2 = sqrt(2);
SARrot = nan(size(SAR));
h = waitbar(0,'Slow implementation with for-loops over rows and cols...', ...
    'name','PolSAR rotate');
for ic=1:ncols
    waitbar(ic/ncols,h);
    for ir=1:nrows
        c2t = cosd(2*theta(ir,ic));
        s2t = sind(2*theta(ir,ic));
        U = 0.5*[  1+c2t  s2*s2t  1-c2t
                 -s2*s2t   2*c2t s2*s2t
                   1-c2t -s2*s2t  1+c2t];
%         if ~isreal(U), fprintf('U complex, ir=%i, ic=%i\n',ir,ic), end
        aux = U*wc2mat(SAR(ir,ic,:))*U.';
        SARrot(ir,ic,1) = real(aux(1,1));
        SARrot(ir,ic,2) = real(aux(1,2));
        SARrot(ir,ic,3) = imag(aux(1,2));
        SARrot(ir,ic,4) = real(aux(1,3));
        SARrot(ir,ic,5) = imag(aux(1,3));
        SARrot(ir,ic,6) = real(aux(2,2));
        SARrot(ir,ic,7) = real(aux(2,3));
        SARrot(ir,ic,8) = imag(aux(2,3));
        SARrot(ir,ic,9) = real(aux(3,3));
    end
end
close(h)
